/*!**************************************************************************
\file    AuxFuncs.h

*****************************************************************************/
#ifndef __AuxFuncs_H__
#define __AuxFuncs_H__

#include <cassert>
#include <cfloat>


//#################################################################################
//! \brief Global variable definitions
//#################################################################################

#define FLOATMAX ((float)(1e15))
#define MAXINT ((int)(32700))
#define EPSILON 3.0e-7
#define PI 3.14159265358979323846

#ifdef _MSC_VER
#define log2(A) log10((float)A)/log10(2.0f)
//#define isfinite(A) (!(_isnan(A)))
#endif

//#define isfiniteBTF(A) (!(isnanf(A)))
#define isfiniteBTF(A) (isfinite(A))

inline float luminance(const float RGB[3])
{
  assert(RGB);
  // coefficients to compute luminance from RGB
  const float wlum[3] = {0.299f,  // R
			 0.587f,  // G
			 0.114f}; // B
  float lum = wlum[0] * RGB[0] + wlum[1] * RGB[1] + wlum[2] * RGB[2];
  return lum;  
}

inline double luminance(const double RGB[3])
{
  assert(RGB);
  // coefficients to compute luminance from RGB
  const double wlum[3] = {0.299,  // R
			 0.587,  // G
			 0.114}; // B
  double lum = wlum[0] * RGB[0] + wlum[1] * RGB[1] + wlum[2] * RGB[2];
  return lum;  
}

// from phi and theta to unit 3D vector. Z-axis points to the north pole
// of the sphere
void
ConvertThetaPhiToXYZ(float theta, float phi, float xyz[]);

// from phi and theta to unit 3D vector. Z-axis points to the north pole
// of the sphere
void
ConvertXYZtoThetaPhi(float xyz[], float &theta, float &phi);

// Computes arbitrary normal
void
ArbitraryNormal(const float N[3], float outN[3]);

// statistics record in C++, see Donald Knuth's book
struct SVAR
{
  float statsV; // computed statistics
  int cnt; // the number of samples taken to compute statistics
  double mean; // mean value1
  double M2; // sum for variance1
  // Statistical support
  SVAR() { reset();}

  // add a single sample
  void update(const double newSampleValue) {
    cnt++;
    double delta = newSampleValue - mean;
    mean += delta/(double)cnt;
    M2 += delta*(newSampleValue - mean);
    // temporarily for debugging
    evaluate();
  }
  // add a single zero
  void updateZero() {
    // we count also for zero values
    cnt++;
    double delta = - mean;
    mean += delta/(double)cnt;
    M2 -= delta*mean;
    // temporarily for debugging
    evaluate();
  }
  // Adding zero value
  void updateZeroXtimes(int cntZeros) {
    assert(cntZeros > 0);
    // naive solution - another faster to be developped
    // based on the variance of two random variables should be easy
    for (int i = 0; i < cntZeros; i++) {
      cnt++;
      double delta = - mean;
      mean += delta/(double)cnt;
      M2 += delta*(-mean);      
    }
    // temporarily for debugging
    evaluate();
  }
  void reset() { statsV = 0; cnt = 0; mean=M2=0; }
  // It returns unbised sample variance (so not for finite population)
  double evaluate() {
    return (statsV = (double)M2/((double)cnt - 1));
  }
  // It returns variance for finite population
  double evaluateFP() {
    return (statsV = (double)M2/((double)cnt));
  }
};


#endif // __AuxFuncs_H__
