// ===================================================================
// $Id: $
//
// colorcnv.cpp
//
//     This is the implementation of color conversion functions,
//     it is required for spectral rendering etc.
//
// Initial coding by Vlastimil Havran, December, 2010.

// GOLEM headers
#include "configg.h"
#include "colorcnv.h"

//-------------------------------------------------
// static variables

// The conversion table from Spectral to XYZ space
extern double
SpectralToXYZtable[wavelengthNumberXYZ * 4];

// ---------------------------------------------------------------
// Return the coefficient for conversion from spectral to XYZ.
// Wavelength is given in meters [SI units] should be in the order
// 360e-9 to 740e-9 [m]. Returns false if success, true if the
// input wavelength is out of the range. Sets the values for
// integration correspondingly.
bool
GetCoeffsSpectrumToXYZ(double &coeffX, double &coeffY, double &coeffZ,
		       double wavelength)
{
  // Get it to the range of nanometers
  wavelength *= 1e9;
  const double startWavelength = 360.0;
  const double stopWavelength = 740.0;
  if ( (wavelength < 360.0) ||
       (wavelength > stopWavelength) ) {
    coeffX = coeffY = coeffZ = 0.0;
    return true; // out of range
  }

  double diffWavelength = wavelength - startWavelength;
  int index = (int)diffWavelength;
  double alpha = 1.0 - (diffWavelength - (double)index);
  
  // set the coefficients correctly by linear interpolation
  coeffX =
    SpectralToXYZtable [ 4 * index + 1] * alpha +
    SpectralToXYZtable [ 4 * (index+1) + 1] * (1.0 - alpha);
  coeffY =
    SpectralToXYZtable [ 4 * index + 2] * alpha +
    SpectralToXYZtable [ 4 * (index+1) + 2] * (1.0 - alpha);
  coeffZ =
    SpectralToXYZtable [ 4 * index + 3] * alpha +
    SpectralToXYZtable [ 4 * (index+1) + 3] * (1.0 - alpha);
  
  return false; // OK
}

// Conversion of the spectral color to the artificial primaries XYZ
void
SpectralToXYZ(float &vX, float &vY, float &vZ,  // output
	      float spectralValue[], // input
	      float wavelengthBeg, float wavelengthEnd,
	      int cntWavelengths)
{
  // loop variable
  int i;

  // the wavelength
  // the sum of the integrals
  double sumX, sumY, sumZ, sumYW;
  sumX = sumY = sumZ = sumYW = 0.0;
  
  // for all the wavelenghts in the spectrum
  for (i = 0; i < cntWavelengths; i++) {
    // get the i-th component from the spectrum
    double value = spectralValue[i];
    double wavelength = wavelengthBeg + 
      (float)i/(float)cntWavelengths*(wavelengthEnd-wavelengthBeg);
    wavelength *= 1e-9; // go to the SI units, it is now in meters.
    double coeffX, coeffY, coeffZ;
    // get the coefficients of the spectral curves
    if (!GetCoeffsSpectrumToXYZ(coeffX, coeffY, coeffZ, wavelength)) {
      sumX += value * coeffX;
      sumY += value * coeffY;
      sumZ += value * coeffZ;
      sumYW += coeffY;
    } // if
  } // for i

  // Now convert the value of the integrals to XYZ space
  vX = sumX / sumYW;
  vY = sumY / sumYW;
  vZ = sumZ / sumYW;
  
  return;
}

// Conversion from XYZ to sRGB (standard RGB) color
void
XYZ_to_sRGB(float X, float Y, float Z, // input
	    float &R, float &G, float &B) // output
{
  // Now convert to the sRGB space.

  // This matrix is according the document describing sRGB space:
  // "A Standard Default Color Space for the Internet - sRGB",
  // written by Michael Stokes (HP), Matthew Anderson (Microsoft),
  // Srinivasna Chandrasekar (Mincrosoft), Ricardo Motta (HP),
  // Version 1.10, November 5, 1996.
  static float xyz2srgbmat[3][3] = {
    {3.2410, -1.5374, -0.4986},
    {-0.9692,  1.876,   0.0416},
    {0.0556, -0.204,   1.057}
  };
  
  R =
    xyz2srgbmat[0][0] * X +
    xyz2srgbmat[0][1] * Y +
    xyz2srgbmat[0][2] * Z;
  G =
    xyz2srgbmat[1][0] * X +
    xyz2srgbmat[1][1] * Y +
    xyz2srgbmat[1][2] * Z;
  B =
    xyz2srgbmat[2][0] * X +
    xyz2srgbmat[2][1] * Y +
    xyz2srgbmat[2][2] * Z;

  return;
}


